Package: gridSVG
Title: Export 'grid' Graphics as SVG
Version: 1.7-7
Authors@R: c(person("Paul", "Murrell", role = c("cre", "aut"),
                    email = "paul@stat.auckland.ac.nz"),
             person("Simon", "Potter", role = "aut",
                    email = "simon@sjp.co.nz"))
Description: Functions to export graphics drawn with package grid to SVG
  format.  Additional functions provide access to SVG features that
  are not available in standard R graphics, such as hyperlinks, 
  animation, filters, masks, clipping paths, and gradient and pattern fills.
Imports: grDevices, graphics, utils, methods, grid, jsonlite, XML
Suggests: lattice
License: GPL
NeedsCompilation: no
Packaged: 2025-10-31 01:04:24 UTC; pmur002
Author: Paul Murrell [cre, aut],
  Simon Potter [aut]
Maintainer: Paul Murrell <paul@stat.auckland.ac.nz>
Repository: CRAN
Date/Publication: 2025-10-31 06:10:59 UTC
Built: R 4.6.0; ; 2026-04-25 19:23:38 UTC; unix
