
Key to different versions of test.R:
===================================

(All of these require ../R/svg.R.old)

A = Viewports are <svg> elements with preserveAspectRatio="none"
    Locations and dimensions are mixture of relative and %

B = Viewports are <g> elements with transforms
    Locations and dimensions have to be ONLY relative

C = Viewports are <svg> elements with preserveAspectRatio="xMinYMin"
    Locations and dimensions are mixture of relative and %

D = Viewports are <svg> elements with preserveAspectRatio="xMinYMin"
    Locations and dimensions are ALL %